#!/usr/bin/python3
"""
// © University of Southampton IT Innovation Centre, 2018
//
// Copyright in this software belongs to University of Southampton
// IT Innovation Centre of Gamma House, Enterprise Road,
// Chilworth Science Park, Southampton, SO16 7NS, UK.
//
// This software may not be used, sold, licensed, transferred, copied
// or reproduced in whole or in part in any manner or form or in or
// on any media by any person other than in accordance with the terms
// of the Licence Agreement supplied with the software, or otherwise
// without the prior written consent of the copyright owners.
//
// This software is distributed WITHOUT ANY WARRANTY, without even the
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
// PURPOSE, except where stated in the Licence Agreement supplied with
// the software.
//
//      Created By :            Nikolay Stanchev
//      Created Date :          02-07-2018
//      Created for Project :   FLAME
"""


import transaction
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import scoped_session, sessionmaker
from zope.sqlalchemy import ZopeTransactionExtension


DBSession = scoped_session(sessionmaker(extension=ZopeTransactionExtension()))  # initialise a ORM session, ought to be reused across the different modules


class ORMClass(object):
    """
    Declares a parent class for all models which eases querying
    """

    @classmethod
    def query(cls):
        """
        Pass down the class name when using the DBSession.query method and use ModelClass.query() instead of DBSession.query(ModelClass)

        :return: the query result object
        """

        global DBSession

        return DBSession.query(cls)

    @staticmethod
    def add(instance):
        """
        Adds an instance of a model to the database.

        :param instance: the instance to be created in the db.
        """

        global DBSession

        with transaction.manager:
            DBSession.add(instance)

    @staticmethod
    def delete(instance):
        """
        Deletes an instance of a model from the database.

        :param instance: the instance to be deleted from the db.
        """

        global DBSession

        with transaction.manager:
            DBSession.delete(instance)

    @staticmethod
    def replace(old_instance, new_instance):
        """
        Replaces an instance of a model from the database with a new instance.

        :param old_instance: the instance to be replaced from the db.
        :param new_instance: the new instance
        """

        global DBSession

        with transaction.manager:
            DBSession.add(new_instance)
            DBSession.delete(old_instance)

    @classmethod
    def delete_all(cls):
        """
        Deletes all instances of a model from the database.
        """

        global DBSession

        with transaction.manager:
            deleted_rows = DBSession.query(cls).delete()

        return deleted_rows


Base = declarative_base(cls=ORMClass)  # initialise a declarative Base instance to use for the web app models (inherits from the base ORM class defined above)
